<?php
/**
 * Plugin Name: Do Not Copy
 * Plugin URI: https://gptblog.jp/do-not-copy
 * Description: 記事・固定ページごとにコピー防止を設定し、管理画面で <pre> タグのコピー許可を設定可能なプラグイン。
 * Version: 1.1.0
 * Author: Toshikazu Hirano
 * License: GPL2
 */

// 設定メニューを追加
function wcp_add_admin_menu() {
    add_options_page('Do Not Copy', 'Do Not Copy', 'manage_options', 'do-not-copy', 'wcp_settings_page');
}
add_action('admin_menu', 'wcp_add_admin_menu');

// 設定ページの内容
function wcp_settings_page() {
    ?>
    <div class="wrap">
        <h2>Do Not Copy 設定</h2>
        <form method="post" action="options.php">
            <?php
            settings_fields('wcp_settings_group');
            do_settings_sections('do-not-copy');
            submit_button();
            ?>
        </form>
    </div>
    <?php
}

// 設定登録
function wcp_register_settings() {
    register_setting('wcp_settings_group', 'wcp_global_copy_protection', 'sanitize_key'); // サニタイズを追加

    add_settings_section('wcp_main_section', '基本設定', null, 'do-not-copy');

    add_settings_field('wcp_global_copy_protection', '全ページのコピー防止', 'wcp_global_copy_field', 'do-not-copy', 'wcp_main_section');
}
add_action('admin_init', 'wcp_register_settings');

// 全ページのコピー防止設定
function wcp_global_copy_field() {
    $value = get_option('wcp_global_copy_protection', '0');
    echo '<input type="checkbox" name="wcp_global_copy_protection" value="1" ' . checked(1, $value, false) . '> 有効';
}

// 記事ごとのコピー防止設定（メタボックス追加）
function wcp_add_meta_box() {
    add_meta_box('wcp_meta_box', 'コピー防止設定', 'wcp_meta_box_callback', ['post', 'page'], 'side');
}
add_action('add_meta_boxes', 'wcp_add_meta_box');

function wcp_meta_box_callback($post) {
    $value = get_post_meta($post->ID, '_wcp_copy_protection', true);
    wp_nonce_field('wcp_meta_box', 'wcp_meta_box_nonce');
    echo '<label><input type="checkbox" name="wcp_copy_protection" value="1" ' . checked(1, $value, false) . '> このページでコピーを防止する</label>';
}

// 記事のメタデータ保存
function wcp_save_post_meta($post_id) {
    if (!isset($_POST['wcp_meta_box_nonce']) || !wp_verify_nonce($_POST['wcp_meta_box_nonce'], 'wcp_meta_box')) return;
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) return;
    if (!current_user_can('edit_post', $post_id)) return;

    // チェックボックスの値をサニタイズ
    $value = isset($_POST['wcp_copy_protection']) ? 1 : 0;
    update_post_meta($post_id, '_wcp_copy_protection', $value);
}
add_action('save_post', 'wcp_save_post_meta');

// コピー防止スクリプトを追加
function wcp_protect_content() {
    $global_enabled = get_option('wcp_global_copy_protection', '0'); // 全ページのコピー防止設定を取得

    if (is_singular()) {
        global $post;
        $enabled = get_post_meta($post->ID, '_wcp_copy_protection', true);
    } else {
        $enabled = false;
    }

    if ($enabled || $global_enabled) { // 記事ごとまたは全体で有効なら適用
        ?>
        <script>
            document.addEventListener("DOMContentLoaded", function() {
                let devToolsOpen = false;

                // 右クリック禁止
                document.addEventListener("contextmenu", function(e) {
                    alert("このコンテンツはコピーできません。");
                    e.preventDefault();
                });

                // テキスト選択禁止
                document.addEventListener("selectstart", function(e) {
                    alert("このコンテンツはコピーできません。");
                    e.preventDefault();
                });

                // コピーキー（Ctrl+C）を無効化
                document.addEventListener("keydown", function(e) {
                    if (e.ctrlKey && (e.key === 'c' || e.key === 'x' || e.key === 'u')) {
                        alert("このコンテンツはコピーできません。");
                        e.preventDefault();
                    }
                });

                // デベロッパーツール対策（F12やCtrl+Shift+Iのブロック）
                document.addEventListener("keydown", function(e) {
                    if (e.keyCode === 123 || (e.ctrlKey && e.shiftKey && e.key === 'I')) {
                        alert("開発者ツールは無効化されています。");
                        e.preventDefault();
                    }
                });

                // デベロッパーツールの監視
                setInterval(function() {
                    if (!devToolsOpen && (window.outerHeight - window.innerHeight > 100 || window.outerWidth - window.innerWidth > 100)) {
                        devToolsOpen = true;
                    }
                }, 1000);
            });
        </script>
        <style>
            /* ドラッグ禁止 */
            body {
                -webkit-user-select: none;
                -moz-user-select: none;
                -ms-user-select: none;
                user-select: none;
            }
        </style>
        <?php
    }
}
add_action('wp_footer', 'wcp_protect_content');
